#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "defines.h"
#include "errormsg.h"
#include "filearea.h"
#include "getdir.h"

extern Widget top_level;
extern char *home_dir;
extern char current_wd[120];

/* TextField widget needed in multiple functions for dir_create_func(). */
Widget dir_c_tf;
/* mode to use when creating directory */
mode_t dir_create_mode;

/* Cancel an operation by destroying its popup shell.  A very useful
 * function */
void chdir_goto_cancelCB(Widget w, XtPointer client_data, XtPointer call_data)
{
  XtDestroyWidget((Widget)client_data);
}

/* Adjust dir_create_mode when the toggles are clicked. */
void toggleCB(Widget w, XtPointer client_data, XtPointer call_data)
{
  Boolean IsSet;

  XtVaGetValues(w, XmNset, &IsSet, NULL);

  if (IsSet)
    dir_create_mode|=(int)client_data;
  else
    dir_create_mode^=(int)client_data;
}

/* Actually create the directory. */
void dir_createCB(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *thisnewdir, fulldirpath[120];

  XtDestroyWidget((Widget) client_data);
  XtVaGetValues(dir_c_tf, XmNvalue, &thisnewdir, NULL);
  sprintf(fulldirpath, "%s/%s", current_wd, thisnewdir);
#ifdef DEBUG
  printf("%d: %s\n", dir_create_mode, fulldirpath);
#endif
  mkdir(fulldirpath, dir_create_mode);

  if (errno == EEXIST) {
    NoCanDo(IT_ALREADY_EXISTS);
    return;
  }

  XtFree(thisnewdir);
  load_dir(current_wd);
  DrawFileArea();
}

/* Create a new directory.  Offer option of changing permissions, etc. */
void dir_create_func() {
  Widget dir_c_shell, dir_c_bb, dir_l1, dir_l2, dir_l3;
  Widget dir_ok, dir_cancel, dir_help;
  Widget uperms, uperms_label, uperms_read, uperms_write, uperms_execute;
  Widget gperms, gperms_label, gperms_read, gperms_write, gperms_execute;
  Widget aperms, aperms_label, aperms_read, aperms_write, aperms_execute;
  struct stat buf;
  XmString xlabelstr;
  Arg args[8];

  stat(current_wd, &buf);
  if (!IsWritable(buf)) {
    NoCanDo(CANT_WRITE_DIR);
    return;
  }

  dir_create_mode = 493;  /* results in: drwxr-xr-x on new directory */

  dir_c_shell = XtVaCreatePopupShell("dir_shell", vendorShellWidgetClass,
				     top_level, XmNwidth, 320, XmNtitle,
				     "Create New Subdirectory",
				     XmNheight, 290, NULL);
  dir_c_bb = XtVaCreateManagedWidget("dir_bb", xmBulletinBoardWidgetClass,
				     dir_c_shell, NULL);
  dir_l1 = XtVaCreateManagedWidget("dir_l1", xmLabelWidgetClass, dir_c_bb,
				   XmNx, 10, XmNy, 0, XmNlabelString,
				   XmStringCreateSimple("Current directory:"),
				   NULL);
  dir_l2 = XtVaCreateManagedWidget("dir_l2", xmLabelWidgetClass, dir_c_bb,
				   XmNx, 10, XmNy, 30, XmNlabelString,
				   XmStringCreateSimple(current_wd),
				   NULL);
  dir_l3 = XtVaCreateManagedWidget("dir_l3", xmLabelWidgetClass, dir_c_bb,
				   XmNx, 10, XmNy, 60, XmNlabelString,
				   XmStringCreateSimple("Name of new subdirectory:"),
				   NULL);
  dir_c_tf = XtVaCreateManagedWidget("dir_c_tf", xmTextFieldWidgetClass, 
				     dir_c_bb,
				     XmNx, 5, XmNy, 85, XmNwidth, 300, NULL);
  XtAddCallback(dir_c_tf, XmNactivateCallback, dir_createCB,
		(XtPointer) dir_c_shell);

  /* Display checkboxes to allow changing permissions. */
  XtSetArg(args[0], XmNorientation, XmVERTICAL);
  XtSetArg(args[1], XmNx, 0);
  XtSetArg(args[2], XmNy, 120);
  uperms=XmCreateRowColumn(dir_c_bb, "info_rc1", args, 3);
  uperms_label=XmCreateLabel(uperms, "User", NULL, 0);
  xlabelstr=XmStringCreateSimple("Read");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  uperms_read=(Widget)XmCreateToggleButton(uperms, "read", args, 1);
  XtAddCallback(uperms_read, XmNvalueChangedCallback, toggleCB, 
		(XtPointer)256);
  xlabelstr=XmStringCreateSimple("Write");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  uperms_write=(Widget)XmCreateToggleButton(uperms, "write", args, 1);
  XtAddCallback(uperms_write, XmNvalueChangedCallback, toggleCB,
		(XtPointer)128);
  xlabelstr=XmStringCreateSimple("Execute");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  uperms_execute=(Widget)XmCreateToggleButton(uperms, "execute", args, 1);
  XtAddCallback(uperms_execute, XmNvalueChangedCallback, toggleCB,
		(XtPointer)64);
  XtVaSetValues(uperms_read, XmNset, True, NULL);
  XtVaSetValues(uperms_write, XmNset, True, NULL);
  XtVaSetValues(uperms_execute, XmNset, True, NULL);
  XtManageChild(uperms);
  XtManageChild(uperms_label);
  XtManageChild(uperms_read);
  XtManageChild(uperms_write);
  XtManageChild(uperms_execute);

  XtSetArg(args[0], XmNorientation, XmVERTICAL);
  XtSetArg(args[1], XmNx, 100);
  XtSetArg(args[2], XmNy, 120);
  gperms=XmCreateRowColumn(dir_c_bb, "info_rc2", args, 3);
  gperms_label=XmCreateLabel(gperms, "Group", NULL, 0);
  xlabelstr=XmStringCreateSimple("Read");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  gperms_read=(Widget)XmCreateToggleButton(gperms, "read", args, 1);
  XtAddCallback(gperms_read, XmNvalueChangedCallback, toggleCB,
		(XtPointer)32);
  xlabelstr=XmStringCreateSimple("Write");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  gperms_write=(Widget)XmCreateToggleButton(gperms, "write", args, 1);
  XtAddCallback(gperms_write, XmNvalueChangedCallback, toggleCB,
		(XtPointer)16);
  xlabelstr=XmStringCreateSimple("Execute");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  gperms_execute=(Widget)XmCreateToggleButton(gperms, "execute", args, 1);
  XtAddCallback(gperms_execute, XmNvalueChangedCallback, toggleCB,
		(XtPointer)8);
  XtVaSetValues(gperms_read, XmNset, True, NULL);
  XtVaSetValues(gperms_write, XmNset, False, NULL);
  XtVaSetValues(gperms_execute, XmNset, True, NULL);
  XtManageChild(gperms);
  XtManageChild(gperms_label);
  XtManageChild(gperms_read);
  XtManageChild(gperms_write);
  XtManageChild(gperms_execute);

  XtSetArg(args[0], XmNorientation, XmVERTICAL);
  XtSetArg(args[1], XmNx, 200);
  XtSetArg(args[2], XmNy, 120);
  aperms=XmCreateRowColumn(dir_c_bb, "info_rc3", args, 3);
  aperms_label=XmCreateLabel(aperms, "Others", NULL, 0);
  xlabelstr=XmStringCreateSimple("Read");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  aperms_read=(Widget)XmCreateToggleButton(aperms, "read", args, 1);
  XtAddCallback(aperms_read, XmNvalueChangedCallback, toggleCB,
		(XtPointer)4);
  xlabelstr=XmStringCreateSimple("Write");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  aperms_write=(Widget)XmCreateToggleButton(aperms, "write", args, 1);
  XtAddCallback(aperms_write, XmNvalueChangedCallback, toggleCB,
		(XtPointer)2);
  xlabelstr=XmStringCreateSimple("Execute");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  aperms_execute=(Widget)XmCreateToggleButton(aperms, "execute", args, 1);
  XtAddCallback(aperms_execute, XmNvalueChangedCallback, toggleCB,
		(XtPointer)1);
  XtVaSetValues(aperms_read, XmNset, True, NULL);
  XtVaSetValues(aperms_write, XmNset, False, NULL);
  XtVaSetValues(aperms_execute, XmNset, True, NULL);
  XtManageChild(aperms);
  XtManageChild(aperms_label);
  XtManageChild(aperms_read);
  XtManageChild(aperms_write);
  XtManageChild(aperms_execute);

  dir_ok = XtVaCreateManagedWidget("Create", xmPushButtonWidgetClass, dir_c_bb,
				   XmNx, 20, XmNy, 250, XmNheight, 30,
				   XmNwidth, 80, NULL);
  XtAddCallback(dir_ok, XmNactivateCallback, dir_createCB,
		(XtPointer) dir_c_shell);
  dir_cancel = XtVaCreateManagedWidget("Cancel", xmPushButtonWidgetClass, 
				       dir_c_bb, XmNx, 120, XmNy, 250, 
				       XmNheight, 30, XmNwidth, 80, NULL);
  XtAddCallback(dir_cancel, XmNactivateCallback, chdir_goto_cancelCB,
		(XtPointer) dir_c_shell);
  dir_help = XtVaCreateManagedWidget("Help", xmPushButtonWidgetClass, dir_c_bb,
				     XmNx, 220, XmNy, 250, XmNheight, 30,
				     XmNwidth, 80, NULL);
  XtAddCallback(dir_help, XmNactivateCallback, chdir_goto_cancelCB,
		(XtPointer) dir_c_shell);
  XtPopup(dir_c_shell, XtGrabNone);

  XmStringFree(xlabelstr);
}

void chdir_home() {
  load_dir(home_dir);
  DrawFileArea();
}

void chdir_up() {
  load_dir(getparent(current_wd));
  DrawFileArea();
}

Widget goto_tf;

void chdir_goto_okCB(Widget w, XtPointer client_data, XtPointer call_data)
{
  struct stat buf;
  char *thisdir;

  XtDestroyWidget((Widget)client_data);
  XtVaGetValues(goto_tf, XmNvalue, &thisdir, NULL);
  stat(thisdir, &buf);
  if (errno == ENOENT) {
    NoCanDo(DIR_NOT_EXIST);
    return;
  }
  if (!IsReadable(buf)) {
    NoCanDo(CANT_READ_DIR);
    return;
  }
  if (!S_ISDIR(buf.st_mode)) {
    NoCanDo(NOT_A_DIRECTORY);
    return;
  }

  load_dir(thisdir);
  DrawFileArea();
}
  
void chdir_goto() {
  Widget goto_shell, goto_bb, goto_label, goto_ok, goto_cancel, goto_help;
  XmString goto_msg;

  goto_msg = XmStringCreateSimple("Change to this directory:");
  
  goto_shell = XtVaCreatePopupShell("goto_shell", vendorShellWidgetClass,
				    top_level, XmNtitle, "Change directory",
				    XmNwidth, 280, XmNheight, 125, NULL);
  goto_bb = XtVaCreateManagedWidget("goto_bb", xmBulletinBoardWidgetClass,
				    goto_shell, XmNwidth, 280, 
				    XmNheight, 125, NULL);
  goto_label = XtVaCreateManagedWidget("goto_label", xmLabelWidgetClass,
				       goto_bb, XmNlabelString, goto_msg,
				       XmNx, 5, XmNy, 5, NULL);
  goto_tf = XtVaCreateManagedWidget("goto_tf", xmTextFieldWidgetClass,
				    goto_bb, XmNx, 5, XmNy, 35, 
				    XmNwidth, 260, NULL);
  XtAddCallback(goto_tf, XmNactivateCallback, chdir_goto_okCB,
		(XtPointer) goto_shell);
  goto_ok = XtVaCreateManagedWidget("OK", xmPushButtonWidgetClass, goto_bb,
				    XmNx, 25, XmNy, 80, XmNheight, 30,
				    XmNwidth, 60, NULL);
  XtAddCallback(goto_ok, XmNactivateCallback, chdir_goto_okCB,
		(XtPointer) goto_shell);
  goto_cancel = XtVaCreateManagedWidget("Cancel", xmPushButtonWidgetClass, 
					goto_bb, XmNx, 110, XmNy, 80, 
					XmNheight, 30, XmNwidth, 60, NULL);
  XtAddCallback(goto_cancel, XmNactivateCallback, chdir_goto_cancelCB,
		(XtPointer) goto_shell);
  goto_help = XtVaCreateManagedWidget("Help", xmPushButtonWidgetClass, goto_bb,
				      XmNx, 195, XmNy, 80, XmNheight, 30,
				      XmNwidth, 60, NULL);
  XtAddCallback(goto_help, XmNactivateCallback, chdir_goto_cancelCB,
		(XtPointer) goto_shell);

  XtPopup(goto_shell, XtGrabNone);
  XmStringFree(goto_msg);
}
				    
  
